import java.awt.*;
import java.awt.event.*;

import javax.swing.event.*;
import javax.swing.*;

import java.util.*;
import java.math.*;

public class Controller extends JApplet implements ActionListener,
		AdjustmentListener, ChangeListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	// Parameter (alle in SI Einheiten)
	// Nur PKW Parameter per Slider verstellbar!
	static double tau = 0.1; // Zeitskalierung, man kann hiermit das Programm
								// beschleunigen.
	static int L = 3; // Gesmatzahl der Spuren
	static double dt = 2; // Zeitlicher Abstand
	static int n_veh = 35; // Gesamtzahl der Fahrzeuge
	static int n_pkw = 30; // Anzahl der PKWs
	static int n_lkw = 0; // Anzahl der LKWs
	static double h = 0.001; // Zeitschritt fuer den DGL solver
	static double a_pkw = 0.3; // Beschleunigung
	static double b_pkw = 2; // angenehme Bremsbeschleunigung
	static double s_ref_pkw = 6; // minimaler Abstand
	static double v_ref_pkw = 120 / 3.6; // bevorzugte Geschwindigkeit
	static double a_lkw = 0.3; // Beschleunigung
	static double b_lkw = 3.5; // angenehme Bremsbeschleunigung
	static double s_ref_lkw = 15; // minimaler Abstand
	static double v_ref_lkw = 100 / 3.6; // bevorzugte Geschwindigkeit
	static double delta = 4; // komische Konstante
	static double length_lkw = 6; // Fahrzeuglnge
	static double length_pkw = 3; // Fahrzeuglnge
	static double framerate = 900; // Framerate in Hz
	static int repaintRate = 1; // Neuzeichnen der Oberflche alle repaintRate
								// Berechnungen

	// Anfangsbedingungen
	static double x0_pkw = 15; // anfnglicher Abstand
	static double v0_pkw = 120 / 3.6; // anfngliche Geschwindigkeit
	static double x0_lkw = 15; // anfnglicher Abstand
	static double v0_lkw = 100 / 3.6; // anfngliche Geschwindigkeit
	static double rand = 10; // maximale Schwankung des Abstandes

	// Sonstige
	static int winwidth; // Breite des Appletfensters
	static int winheight; // Hoehe des Appletfensters
	static boolean pause = false; // Programm pausieren
	static boolean reset = false; // Variablen zurcksetzen
	static ArrayList[] vehicleArray = { new ArrayList<Vehicle>(),
			new ArrayList<Vehicle>(), new ArrayList<Vehicle>() };

	JFrame window = null;

	// Oberflche
	// buttons and sliders
	static JButton bStart = new JButton("Start");
	static JButton bPause = new JButton("Pause");
	static JButton bReset = new JButton("Reset");

	/*
	 * here, the sliders are defined. first, they are initialized with their
	 * orientation, the min and max values and the initial value displayed. the
	 * initial value could be used to implement a preset.
	 */

	JSlider sDesiredDistances0 = new JSlider(JSlider.VERTICAL, 0, 200,
			(int) (s_ref_pkw * 10));
	JSlider sDesiredSpeedv0 = new JSlider(JSlider.VERTICAL, 0, 1600,
			(int) (v_ref_pkw * 36));
	JSlider sMaximalAcellerationA = new JSlider(JSlider.VERTICAL, 0, 50,
			(int) (a_pkw * 10));
	JSlider sComfortAcellerationB = new JSlider(JSlider.VERTICAL, 0, 50,
			(int) b_pkw * 10);
	JSlider sTimeDistance = new JSlider(JSlider.VERTICAL, 0, 50,
			(int) (dt * 10));
	JSlider sDelta = new JSlider(JSlider.VERTICAL, 0, 5, (int) (delta ));
	JSlider sNumberOfCarsn = new JSlider(JSlider.VERTICAL, 0, 50, n_pkw);
	JSlider sNumberOfTrucksp = new JSlider(JSlider.VERTICAL, 0, 25, n_lkw);

	// Funktionen

	// +++++++++++++++++++++++++++++++++ Grafische
	// Oberflche+++++++++++++++++++++++++++++++

	/*
	 * This code shall use a box layout manager to create the general layout.
	 * The centerpart shall contain the simulation itself, the north is occupied
	 * with buttons and south contains scrollbars for parameters.
	 */

	public void DrawSurface(int width, int height, ArrayList[] vehicleArray) {

		this.setSize(width, height); // size

		/*
		 * register Listeners to buttons and sliders
		 */
		bStart.addActionListener(this);
		bPause.addActionListener(this);
		bReset.addActionListener(this);

		sDesiredDistances0.addChangeListener(this);
		sDesiredSpeedv0.addChangeListener(this);
		sMaximalAcellerationA.addChangeListener(this);
		sComfortAcellerationB.addChangeListener(this);
		sTimeDistance.addChangeListener(this);
		sDelta.addChangeListener(this);
		sNumberOfCarsn.addChangeListener(this);
		sNumberOfTrucksp.addChangeListener(this);

		/*
		 * here the maximal size is set, so that the slider does not fill the
		 * whole window
		 */

		sDesiredDistances0.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));
		sDesiredSpeedv0.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));
		sMaximalAcellerationA.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));
		sComfortAcellerationB.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));
		sTimeDistance.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));
		sDelta.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));
		sNumberOfCarsn.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));
		sNumberOfTrucksp.setMaximumSize(new Dimension((int) (width * 0.6),
				Short.MAX_VALUE));

		/*
		 * the labels will be painted
		 */
		sDesiredDistances0.setPaintLabels(true);
		sDesiredSpeedv0.setPaintLabels(true);
		sMaximalAcellerationA.setPaintLabels(true);
		sComfortAcellerationB.setPaintLabels(true);
		sTimeDistance.setPaintLabels(true);
		sDelta.setPaintLabels(true);
		sNumberOfCarsn.setPaintLabels(true);
		sNumberOfTrucksp.setPaintLabels(true);

		/*
		 * some sliders need to give double values, so the labels are renamed
		 */

		Hashtable DeltaTable = new Hashtable();
		DeltaTable.put(new Integer(0), new JLabel("0.0"));
		DeltaTable.put(new Integer(4), new JLabel("1.0"));
		DeltaTable.put(new Integer(8), new JLabel("2.0"));
		DeltaTable.put(new Integer(12), new JLabel("3.0"));
		DeltaTable.put(new Integer(16), new JLabel("4.0"));
		DeltaTable.put(new Integer(20), new JLabel("5.0"));

		sMaximalAcellerationA.setLabelTable(DeltaTable);
		sComfortAcellerationB.setLabelTable(DeltaTable);

		Hashtable DistanceTable = new Hashtable();
		DistanceTable.put(new Integer(0), new JLabel("0"));
		DistanceTable.put(new Integer(50), new JLabel("5"));
		DistanceTable.put(new Integer(100), new JLabel("10"));
		DistanceTable.put(new Integer(150), new JLabel("15"));
		DistanceTable.put(new Integer(200), new JLabel("20"));

		sDesiredDistances0.setLabelTable(DistanceTable);

		Hashtable SpeedTable = new Hashtable();
		SpeedTable.put(new Integer(0), new JLabel("0"));
		SpeedTable.put(new Integer(200), new JLabel("20"));
		SpeedTable.put(new Integer(400), new JLabel("40"));
		SpeedTable.put(new Integer(600), new JLabel("60"));
		SpeedTable.put(new Integer(800), new JLabel("80"));
		SpeedTable.put(new Integer(1000), new JLabel("100"));
		SpeedTable.put(new Integer(1200), new JLabel("120"));
		SpeedTable.put(new Integer(1400), new JLabel("140"));
		SpeedTable.put(new Integer(1600), new JLabel("160"));

		sDesiredSpeedv0.setLabelTable(SpeedTable);

		Hashtable AccTable = new Hashtable();
		AccTable.put(new Integer(0), new JLabel("0"));
		AccTable.put(new Integer(10), new JLabel("1"));
		AccTable.put(new Integer(20), new JLabel("2"));
		AccTable.put(new Integer(30), new JLabel("3"));
		AccTable.put(new Integer(40), new JLabel("4"));
		AccTable.put(new Integer(50), new JLabel("5"));

		sMaximalAcellerationA.setLabelTable(AccTable);
		sComfortAcellerationB.setLabelTable(AccTable);
		sTimeDistance.setLabelTable(AccTable);

		Hashtable CarTable = new Hashtable();
		CarTable.put(new Integer(0), new JLabel("0"));
		CarTable.put(new Integer(10), new JLabel("10"));
		CarTable.put(new Integer(20), new JLabel("20"));
		CarTable.put(new Integer(30), new JLabel("30"));
		CarTable.put(new Integer(40), new JLabel("40"));
		CarTable.put(new Integer(50), new JLabel("50"));
		CarTable.put(new Integer(60), new JLabel("60"));
		sNumberOfCarsn.setLabelTable(CarTable);
		/*
		 * the major and minor tick spacing is adjusted
		 */
		sDesiredDistances0.setMajorTickSpacing(50);
		sDesiredSpeedv0.setMajorTickSpacing(200);
		sMaximalAcellerationA.setMajorTickSpacing(10);
		sComfortAcellerationB.setMajorTickSpacing(10);
		sTimeDistance.setMajorTickSpacing(10);
		sDelta.setMajorTickSpacing(1);
		sNumberOfCarsn.setMajorTickSpacing(5);
		sNumberOfTrucksp.setMajorTickSpacing(5);

		int minorTickSpacing = 1;
		sDesiredDistances0.setMinorTickSpacing(10);
		sDesiredSpeedv0.setMinorTickSpacing(50);
		sMaximalAcellerationA.setMinorTickSpacing(minorTickSpacing);
		sComfortAcellerationB.setMinorTickSpacing(minorTickSpacing);
		sTimeDistance.setMinorTickSpacing(minorTickSpacing);
		//sDelta.setMinorTickSpacing(1);
		sNumberOfCarsn.setMinorTickSpacing(minorTickSpacing);
		sNumberOfTrucksp.setMinorTickSpacing(minorTickSpacing);

		/*
		 * ticks and labels (the numbers below the sliders) are painted
		 */

		sDesiredDistances0.setPaintTicks(true);
		sDesiredSpeedv0.setPaintTicks(true);
		sMaximalAcellerationA.setPaintTicks(true);
		sComfortAcellerationB.setPaintTicks(true);
		sTimeDistance.setPaintTicks(true);
		sDelta.setPaintTicks(true);
		sNumberOfCarsn.setPaintTicks(true);
		sNumberOfTrucksp.setPaintTicks(true);

		sDesiredDistances0.setPaintLabels(true);
		sDesiredSpeedv0.setPaintLabels(true);
		sMaximalAcellerationA.setPaintLabels(true);
		sComfortAcellerationB.setPaintLabels(true);
		sTimeDistance.setPaintLabels(true);
		sDelta.setPaintLabels(true);
		sNumberOfCarsn.setPaintLabels(true);
		sNumberOfTrucksp.setPaintLabels(true);

		// pane for the buttons on top
		JPanel buttonPane = new JPanel(); // for the content
		buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));

		// the horizontal glue defines where unused space should go, as it goes
		// equally left and right,
		// the buttons are centered.
		buttonPane.add(Box.createHorizontalGlue());
		buttonPane.add(bStart);
		buttonPane.add(bPause);
		buttonPane.add(bReset);
		buttonPane.add(Box.createHorizontalGlue());

		// this creates a dimension object to ensure 5 pixels distance between
		// the scrollbars
		int dist = 5;
		Dimension minSize = new Dimension(dist, dist);
		Dimension prefSize = new Dimension(dist, dist);
		Dimension maxSize = new Dimension(Short.MAX_VALUE, dist);

		// pane for the Scrollbars below
		JPanel sliderPanel = new JPanel(); // for the content
		sliderPanel.setLayout(new BoxLayout(sliderPanel, BoxLayout.X_AXIS));
		sliderPanel.add(sDesiredDistances0, BorderLayout.SOUTH);
		sliderPanel.add(sDesiredSpeedv0, BorderLayout.SOUTH);
		sliderPanel.add(sMaximalAcellerationA, BorderLayout.SOUTH);
		sliderPanel.add(sComfortAcellerationB, BorderLayout.SOUTH);
		sliderPanel.add(sTimeDistance, BorderLayout.SOUTH);
		sliderPanel.add(sDelta, BorderLayout.SOUTH);
		sliderPanel.add(sNumberOfCarsn, BorderLayout.SOUTH);
		sliderPanel.add(sNumberOfTrucksp, BorderLayout.SOUTH);

		this.add(sliderPanel);

		/*
		 * panel for the road and the simulation itself:
		 */
		simulationPanel mySimulation = new simulationPanel(width, height, L,
				vehicleArray);
		/*
		 * labels for the sliders, ordered in a 1 times 8 matrix via GridLayout
		 */

		JPanel labelPanel = new JPanel();
		labelPanel.setLayout(new GridLayout(1, 8));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Minimal Distance" + "</html>"));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Desired Speed" + "</html>"));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Max Accelleration" + "</html>"));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Comfort Deccelleration" + "</html>"));
		labelPanel.add(new Box.Filler(minSize, prefSize, maxSize));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Time Distance" + "</html>"));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Delta" + "</html>"));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Number of Cars" + "</html>"));
		labelPanel.add(new JLabel("<html><div style=\"text-align: center;\">"
				+ "Number of Trucks" + "</html>"));
		labelPanel.setBorder(BorderFactory.createLineBorder(Color.black));

		/*
		 * the JPanel temp combines the labelPanel and the sliderPanel
		 */
		JPanel temp = new JPanel();
		temp.setLayout(new BoxLayout(temp, BoxLayout.Y_AXIS));
		temp.add(labelPanel);
		temp.add(sliderPanel);

		// container that contains buttons and scrollbars and puts them at their
		// desired places.
		Container contentPane = this.getContentPane();
		contentPane.add(buttonPane, BorderLayout.PAGE_START);
		contentPane.add(temp, BorderLayout.SOUTH);
		contentPane.add(mySimulation, BorderLayout.CENTER);
		contentPane.validate();

	}// end DrawSurface

	/*
	 * action listeners for the buttons
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == bStart) { // start the simulation
			System.out.println("Start Pressed");
			pause = false;
			try {
				this.notify();
			} catch (Exception e1) {
				// e.printStackTrace();
			}

		}
		if (e.getSource() == bPause) { // pause the animation
			System.out.println("Pause Pressed");
			pause = true;

		}
		if (e.getSource() == bReset) { // reset the animation
			pause = true;
			System.out.println("Reset Pressed");
			try {
				Thread.sleep(500);

			} catch (InterruptedException e3) {
			}

			for (int i = 0; i < L; i++) {
				vehicleArray[i].clear();
			}

			vehicleArray = InitialiseVehicles();
			this.repaint();
		}
	}

	/*
	 * action listeners for the sliders
	 */
	// @Override
	public void stateChanged(ChangeEvent e) {
		if (e.getSource() == sDesiredDistances0) {
			// pause = true;

			if (sDesiredDistances0.getValue() == 0) {
				sDesiredDistances0.setValue(1);
			}
			s_ref_pkw = (double) sDesiredDistances0.getValue() / 10;

			System.out.println("Desired Distance " + s_ref_pkw);

			updatevehicles();
		}
		if (e.getSource() == sDesiredSpeedv0) {
			if (sDesiredSpeedv0.getValue() == 0) {
				sDesiredSpeedv0.setValue(1);
			}
			v_ref_pkw = (double) sDesiredSpeedv0.getValue() / 10 / 3.6;
			System.out.println("Desired Speed" + this.v_ref_pkw);
			v0_pkw = sDesiredSpeedv0.getValue() / 10 / 3.6;
			updatevehicles();
		}
		if (e.getSource() == sMaximalAcellerationA) {
			if (sMaximalAcellerationA.getValue() == 0) {
				sMaximalAcellerationA.setValue(1);
			}
			a_pkw = (double) sMaximalAcellerationA.getValue() / 10;
			System.out.println("Maximal Acceleration " + a_pkw);
			updatevehicles();
		}
		if (e.getSource() == sComfortAcellerationB) {
			if (sComfortAcellerationB.getValue() == 0) {
				sComfortAcellerationB.setValue(1);
			}
			b_pkw = (double) sComfortAcellerationB.getValue() / 10;
			System.out.println("Comfort Acceleration" + b_pkw);
			updatevehicles();
		}
		if (e.getSource() == sTimeDistance) {
			if (sTimeDistance.getValue() == 0) {
				sTimeDistance.setValue(1);
			}
			dt = (double) sTimeDistance.getValue() / 10;
			System.out.println("Time Distance" + dt);
			updatevehicles();

		}
		if (e.getSource() == sDelta) {
			if (sDelta.getValue() == 0) {
				sDelta.setValue(1);
			}
			delta = (double) sDelta.getValue();
			System.out.println("Delta " + delta);
			updatevehicles();
		}
		if (e.getSource() == sNumberOfCarsn) {
			if (sNumberOfCarsn.getValue() == 0) {
				sNumberOfCarsn.setValue(1);
			}
			pause = true;
			if (sNumberOfCarsn.getValue() + n_lkw <= 50) {
				n_pkw = sNumberOfCarsn.getValue();
				System.out.println("Number of Cars" + n_pkw);

				n_veh = n_pkw + n_lkw;

				for (int i = 0; i < L; i++) {
					vehicleArray[i].clear();
				}
				// vehicleArray = new Vehicle[n_veh][this.L];
				vehicleArray = InitialiseVehicles();
			} else {
				sNumberOfCarsn.setValue(50 - n_lkw);
			}
			this.repaint();
		}
		if (e.getSource() == sNumberOfTrucksp) {
			pause = true;
			if (sNumberOfTrucksp.getValue() + n_pkw <= 50) {
				n_lkw = sNumberOfTrucksp.getValue();
				System.out.println("Number of Trucks" + n_lkw);
				n_veh = n_pkw + n_lkw;

				for (int i = 0; i < L; i++) {
					vehicleArray[i].clear();
				}

				vehicleArray = InitialiseVehicles();

			} else {
				sNumberOfTrucksp.setValue(50 - n_pkw);
			}
			this.repaint();
		}
	}

	/*
	 * gets the new value from a changed slider
	 */

	// for drawing the center panel
	public class simulationPanel extends JPanel {
		private ArrayList[] vehicleArray = null;
		int height;
		int width;
		int numberOfLanes;

		public simulationPanel(int width, int height, int numberOfLanes,
				ArrayList[] vehicleArray) {
			this.height = height;
			this.width = width;
			this.numberOfLanes = numberOfLanes;
			this.vehicleArray = vehicleArray;
		}

		public int getHeight() {
			return height;
		}

		public void setHeight(int height) {
			this.height = height;
		}

		public int getWidth() {
			return width;
		}

		public void setWidth(int width) {
			this.width = width;
		}

		public int getNumberOfLanes() {
			return numberOfLanes;
		}

		public void setNumberOfLanes(int numberOfLanes) {
			this.numberOfLanes = numberOfLanes;
		}

		@Override
		public void paint(Graphics g) {
			int widthOfLane = 40;

			// System.out.println("Paint wurde aufgerufen ++++++++++++++++++++++");

			// grass on upper end
			g.setColor(Color.green);
			g.drawRect(0, 0, this.width, 20);
			g.fillRect(0, 0, this.width, 20);

			// grey unterground for the street
			g.setColor(Color.gray);
			g.drawRect(0, 20, this.width, widthOfLane * this.numberOfLanes);
			g.fillRect(0, 20, this.width, widthOfLane * this.numberOfLanes);

			g.setColor(Color.white);

			// upper border lines
			g.fillRect(0, 22, this.width, 2);
			g.fillRect(0, 26, this.width, 2);

			// lane stripes
			for (int j = 1; j < this.numberOfLanes; j++) {

				for (int i = 0; i < (int) (this.width / 50); i++)
					g.fillRect(50 * i, 20 + j * widthOfLane, 20, 2);
			}

			// lower border lines
			g.fillRect(0, 20 + numberOfLanes * widthOfLane - 6, width, 2);
			g.fillRect(0, 20 + numberOfLanes * widthOfLane - 2, width, 2);

			// grass on lower end
			g.setColor(Color.green);
			g.drawRect(0, 20 + numberOfLanes * widthOfLane, width, 20);
			g.fillRect(0, 20 + numberOfLanes * widthOfLane, width, 20);

			// cars
			Vehicle temp;
			// g.setColor(Color.blue);
			int posx, posy;

			for (int j = 0; j < numberOfLanes; j++) {
				posy = 40 * (j + 1);
				for (int i = 0; i < n_veh; i++) {
					temp = (Vehicle) vehicleArray[j].get(i);
					posx = (int) Math.round(temp.getx());
					g.setColor(temp.getColorOfCar());
					g.fillRect(posx - (int) temp.getlength(), posy - 10,
							(int) temp.getlength(), 20);

				}
			}
		}
	}

	@Override
	public void adjustmentValueChanged(AdjustmentEvent e) {

	}

	// +++++++++++++++++++++++++++++++++
	// Steuerung++++++++++++++++++++++++++++++++++++++++++
	public static ArrayList[] InitialiseVehicles() {
		// Initialisiert die Fahrzeuge und definiert die
		// Anfangsbedingungen der Autos und LKWs

		int createdLKW = 0;

		// Initialisiere Fahrzeuge so, dass der Anteil der LKW stimmt

		double p = (double) n_lkw / n_veh; // prozentualer Anteil LKWs
		double lastvehicle;

		for (int j = 0; j < L; j++) {

			lastvehicle = 0;
			createdLKW = 0;

			for (int i = 0; i < n_veh; i++) {
				if ((Math.random() <= p) && (createdLKW < n_lkw)) {
					lastvehicle = lastvehicle + x0_lkw + rand
							* (0.5 - Math.random());
					vehicleArray[j].add(new LKW(i, v_ref_lkw, v0_lkw,
							s_ref_lkw, dt, a_lkw, b_lkw, lastvehicle, L, delta,
							length_lkw));
					createdLKW++;
				} else if (n_pkw + createdLKW <= n_veh) {
					lastvehicle = lastvehicle + x0_pkw + rand
							* (0.5 - Math.random());
					vehicleArray[j].add(new Car(i, v_ref_pkw, v0_pkw,
							s_ref_pkw, dt, a_pkw, b_pkw, lastvehicle, L, delta,
							length_pkw));
				} else {
					lastvehicle = lastvehicle + x0_lkw + rand
							* (0.5 - Math.random());
					vehicleArray[j].add(new LKW(i, v_ref_lkw, v0_lkw,
							s_ref_lkw, dt, a_lkw, b_lkw, lastvehicle, L, delta,
							length_lkw));
					createdLKW++;
				}
			}
		}
		System.out.println("Arraylnge:\t" + vehicleArray[0].size());
		return vehicleArray;
	}

	public void updatevehicles() {
		// Updated vehicleArray nachdem ein Parameter verndert wurde
		Vehicle temp = null;
		for (int j = 0; j < L; j++) {
			for (int i = 0; i < n_veh; i++) {
				temp = (Vehicle) vehicleArray[j].get(i);
				if (temp.getColorOfCar() == Color.blue) {
					temp.seta(a_pkw);
					temp.setb(b_pkw);
					temp.setdelta(delta);
					temp.setdt(dt);
					temp.sets_ref(s_ref_pkw);
					temp.setv_ref(v_ref_pkw);
					// vehicleArray[j].remove(i);
					// vehicleArray[j].add(i, temp);

				}
			}
		}
	}

	// Hauptroutine
	public void init() {

		double[][] positionvelocity = new double[60][2];
		Vehicle[] singleLaneVehicles = new Vehicle[60];
		winwidth = 800;// this.getSize().width;
		winheight = 500;// this.getSize().height;

		// Initialisiere Autos
		vehicleArray = InitialiseVehicles();

		// Initialisiere Oberflche
		DrawSurface(winwidth, winheight, vehicleArray);
		this.setSize(winwidth, winheight);
		this.validate();
		this.setVisible(true);

		// Auf Startbefehl warten
		pause = true;

		while (!Thread.interrupted()) {
			// Abfrage die ggf. pausiert
			// synchronized (this){
			while (this.pause) {
				try {
					this.wait();
				} catch (Exception e) {
				}
			}
			// Ende Pauseblock
			Vehicle temp = null;

			for (int j = 0; j < this.L; j++) {
				// Erstelle einen Array fr die Geschwindigkeiten und Positionen
				// der Autos

				for (int i = 0; i < n_veh; i++) {
					temp = (Vehicle) vehicleArray[j].get(i);
					positionvelocity[i][0] = temp.getx();
					positionvelocity[i][1] = temp.getV();
				}
				// Rufe Update auf
				for (int k = 0; k < n_veh; k++) {
					singleLaneVehicles[k] = (Vehicle) vehicleArray[j].get(k);
				}

				positionvelocity = Model.update(positionvelocity, h,
						singleLaneVehicles);

				// Schreibe Werte in Klasse
				for (int i = 0; i < n_veh; i++) {
					temp = (Vehicle) vehicleArray[j].get(i);
					temp.setx(positionvelocity[i][0]);
					temp.setV(positionvelocity[i][1]);
				}
			}

			this.repaint();
		}
	}

}